'use strict';

Object.defineProperty(exports, '__esModule', {
	value: true
});
exports['default'] = createRenderServer;

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { 'default': obj }; }

var _webpack = require('webpack');

var _webpack2 = _interopRequireDefault(_webpack);

var _child_process = require('child_process');

var _events = require('events');

var _events2 = _interopRequireDefault(_events);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

function createRenderServer(renderer, _ref) {
	var watch = _ref.watch;

	var serverCompiler = (0, _webpack2['default'])(renderer);
	var url = '/';
	var events = new _events2['default']();
	var child = null;
	var addr = null;
	var start = false;
	var port = 0;
	var assetStats = null;
	var serverStats = null;

	function send() {
		if (child) {
			child.send([url, assetStats]);
		}
	}

	function trigger() {
		if (!start || !serverStats) {
			return;
		} else if (child) {
			// If we're already running just invoke HMR, otherwise start up.
			child.kill('SIGUSR2');
		} else {
			(function () {

				var map = serverStats.toJson({ assets: true }).assetsByChunkName;
				var modules = Object.keys(map).map(function (key) {
					return _path2['default'].join(renderer.output.path, map[key]);
				});

				// Only support one entrypoint right now. Maybe support more later.
				if (modules.length !== 1) {
					throw new Error('Must only export 1 entrypoint!');
				}

				child = (0, _child_process.fork)(modules[0], [], {
					env: { PORT: port }
				});
				child.on('message', function (_addr) {
					addr = _addr;
					events.emit('listening');
				});
				child.once('exit', function () {
					events.emit('close');
				});
				child.once('error', function () {
					events.emit('close');
				});
				send();
			})();
		}
	}

	serverCompiler.watch({}, function (err, _stats) {
		// Bail on failure.
		if (err) {
			return;
		};
		serverStats = _stats;
		console.log(serverStats.toString(watch.stats));
		trigger();
	});

	return Object.assign(events, {
		compiler: serverCompiler,
		listen: function listen(_port) {
			start = true;
			port = _port;
			trigger();
		},
		address: function address() {
			return addr;
		},
		assets: function assets(_url) {
			url = _url;
			send();
		},
		stats: function stats(_stats) {
			assetStats = _stats.toJson({
				hash: true,
				version: false,
				timings: false,
				assets: false,
				chunks: true,
				chunkModules: false,
				modules: false,
				cached: false,
				reasons: false,
				source: false,
				errorDetails: false,
				chunkOrigins: false
			});
			send();
		}
	});
}

module.exports = exports['default'];
//# sourceMappingURL=renderer.js.map